#!/usr/bin/env python

"""monchat server program."""

# David Fifield, 2007

import socket

import clienthandler
from util import debug_print, error, format_address

SERVER_IP = ''
PORT = 2001
SERVER_ADDRESS = (SERVER_IP, PORT)

def main():
	"""Run the main program.
	
	This opens a socket, receives connections, and dispatches them to
	client handling threads.
	"""

	debug_print("Opening listener socket.")
	try:
		listener_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		# Allow reusing local addresses. See
		# http://michael.toren.net/mirrors/sock-faq/unix-socket-faq-4.html#ss4.5.
		listener_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
		listener_socket.bind(SERVER_ADDRESS)
		listener_socket.listen(1)
	except socket.error, (errno, message):
		error("Couldn't open socket: " + message + ".")
		return

	try:
		while True:
			(connection, address) = listener_socket.accept()
			debug_print("Accepted connection from " + format_address(address))
			clienthandler.ClientHandler(connection, address).start()
	except KeyboardInterrupt:
		debug_print("Caught keyboard interrupt.")
	finally:
		debug_print("Closing listener socket.")
		listener_socket.close()
		for client in clienthandler.client_list:
			client.join()

if __name__ == "__main__":
	main()
